//
//  GameScene.swift
//

import UIKit
import SpriteKit

//移動方向
enum Direction: Int {
	case Right		= 0		//右
	case Left		= 1		//左
}

enum NodeName: String {
	
	case frame_ground = "frame_ground"	//地面あたり
	case frame_floor = "frame_floor"	//浮床あたり
	case player = "player"				//プレイヤー
	case backGround = "backGround"		//背景
	case ground = "ground"				//地面
	case floor = "floor"				//浮床
	
	//接触カテゴリビットマスク
	func category()->UInt32{
		switch self {
		case .frame_ground:	//地面あたり
			return 0x00000001 << 0
		case .frame_floor:	//浮床あたり
			return 0x00000001 << 1
		case .player:		//プレイヤー
			return 0x00000001 << 2
		default:
			return 0x00000000
		}
	}
}


class GameScene: SKScene, SKPhysicsContactDelegate {
	
	//画面
	let baseNode = SKNode()						//ゲームベースノード
	let backScrNode = SKNode()					//背景ノード
	var allScreenSize = CGSizeMake(0, 0)		//全画面サイズ
	let oneScreenSize = CGSizeMake(375, 667)	//１画面サイズ
	
	// プレイヤーキャラ
	var playerNode: SKSpriteNode!				
	var playerDirection: Direction = .Right		//移動方向
	var physicsRadius: CGFloat = 14.0			//物理半径
	var playerAcceleration: CGFloat = 50.0		//移動加速値
	var playerMaxVelocity: CGFloat = 200.0		//MAX移動値
	var jumpForce: CGFloat = 16.0				//ジャンプ力
	var charXOffset: CGFloat = 0				//X位置のオフセット
	var charYOffset: CGFloat = 0				//Y位置のオフセット
	var moving: Bool = false					//移動中フラグ
	var jumping: Bool = false					//ジャンプ中フラグ
	var falling: Bool = false					//落下中
	
	var tapPoint: CGPoint = CGPointZero
	var screenSpeed: CGFloat = 12.0
	var screenSpeedScale: CGFloat = 1.0
	
	
	//MARK: シーンが表示されたときに呼ばれる関数
	override func didMoveToView(view: SKView) {
		
		self.backgroundColor = SKColor.clearColor()
		
		//接触デリゲート
		self.physicsWorld.contactDelegate = self
		
		//MARK: 背景
		//地面、キャラのスクロール
		self.addChild(self.baseNode)
		//背景のスクロール
		self.addChild(self.backScrNode)
		
		
		let wCount = 4		//横の画面数
		self.allScreenSize = CGSizeMake(self.oneScreenSize.width * CGFloat(wCount), self.size.height)
		
		//シーンファイルを読み込み
		if let scene = SKScene(fileNamed: "GameScene.sks") {
			//===================
			//背景
			//===================
			scene.enumerateChildNodesWithName("back_wall", usingBlock: { (node, stop) -> Void in
				let back_wall = node as! SKSpriteNode
				back_wall.name = NodeName.backGround.rawValue
				//シーンから削除して再配置
				back_wall.removeFromParent()
				self.backScrNode.addChild(back_wall)
			})
			//===================
			//地面
			//===================
			scene.enumerateChildNodesWithName("ground", usingBlock: { (node, stop) -> Void in
				let ground = node as! SKSpriteNode
				ground.name = NodeName.ground.rawValue
				//シーンから削除して再配置
				ground.removeFromParent()
				self.baseNode.addChild(ground)
			})
			//===================
			//浮床
			//===================
			scene.enumerateChildNodesWithName("floor", usingBlock: { (node, stop) -> Void in
				let floor = node as! SKSpriteNode
				floor.name = NodeName.floor.rawValue
				//シーンから削除して再配置
				floor.removeFromParent()
				self.baseNode.addChild(floor)
			})
			//===================
			//MARK: プレイヤー
			//===================
			self.playerDirection = .Right
			self.charXOffset = self.oneScreenSize.width * 0.5
			self.charYOffset = self.oneScreenSize.height * 0.5
			scene.enumerateChildNodesWithName("player", usingBlock: { (node, stop) -> Void in
				let player = node as! SKSpriteNode
				self.playerNode = player
				//シーンから削除して再配置
				player.removeFromParent()
				self.baseNode.addChild(self.playerNode)
				//物理設定
				self.playerNode.physicsBody = SKPhysicsBody(circleOfRadius: self.physicsRadius,center: CGPointMake(0, self.physicsRadius))
				self.playerNode.physicsBody!.friction = 1.0			//摩擦
				self.playerNode.physicsBody!.allowsRotation = false	//回転禁止
				self.playerNode.physicsBody!.restitution = 0.0		//跳ね返り値
				self.playerNode.physicsBody!.categoryBitMask = NodeName.player.category()
				self.playerNode.physicsBody!.collisionBitMask = NodeName.frame_ground.category()|NodeName.frame_floor.category()
				self.playerNode.physicsBody!.contactTestBitMask = 0
				self.playerNode.physicsBody!.usesPreciseCollisionDetection = true
			})
			
			//===================
			//壁あたり
			//===================
			let wallFrameNode = SKNode()
			self.baseNode.addChild(wallFrameNode)
			//読み込んだシーンのサイズから外周のあたりを作成する
			wallFrameNode.physicsBody = SKPhysicsBody(edgeLoopFromRect: CGRectMake(0, 0, scene.size.width, scene.size.height))
			wallFrameNode.physicsBody!.categoryBitMask = NodeName.frame_ground.category()
			wallFrameNode.physicsBody!.usesPreciseCollisionDetection = true
		}
		
		
	}
	
	//MARK: - 移動
	func moveToRight() {
		self.moving = true					//移動中フラグON
		self.playerDirection = .Right
		if self.jumping == false && self.falling == false {
			let names = ["right2", "right1","right3","right1"]
			self.startTextureAnimation(self.playerNode, names: names)
		} else {
			self.playerNode.texture = SKTexture(imageNamed: "right_jump1")
		}
	}
	func moveToLeft() {
		self.moving = true					//移動中フラグON
		self.playerDirection = .Left
		if self.jumping == false && self.falling == false {
			let names = ["left2", "left1","left3","left1"]
			self.startTextureAnimation(self.playerNode, names: names)
		} else {
			self.playerNode.texture = SKTexture(imageNamed: "left_jump1")
		}
	}
	//MARK: - 停止
	func moveStop() {
		self.moving = false					//移動中フラグOFF
		if self.jumping == false && self.falling == false {
			var name: String!
			if self.playerDirection == .Right {
				name = "right1"
			}
			else {
				name = "left1"
			}
			self.stopTextureAnimation(self.playerNode, name: name)
			self.playerNode.physicsBody!.velocity = CGVectorMake(0, 0)
		}
	}
	//MARK: - ジャンプ
	func jumpingAction() {
		
		if self.jumping == false && self.falling == false {
			self.moving = false
			self.jumping = true
			
			//地面に接触判定
			self.playerNode.physicsBody!.collisionBitMask = NodeName.frame_ground.category()
			self.playerNode.physicsBody!.contactTestBitMask = 0
			
			if self.playerDirection == .Left {
				self.stopTextureAnimation(self.playerNode, name: "left_jump1")
				self.playerNode.physicsBody!.applyImpulse(CGVectorMake(0.0, self.jumpForce))
			}
			else {
				self.stopTextureAnimation(self.playerNode, name: "right_jump1")
				self.playerNode.physicsBody!.applyImpulse(CGVectorMake(0.0, self.jumpForce))
			}
		}
	}
	
	
	
	//MARK: - タッチ処理
	//タッチダウンされたときに呼ばれる関数
	override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
		
		var location: CGPoint!
		for touch in touches {
			location = touch.locationInNode(self)
		}
		self.tapPoint = location
		self.playerNode.physicsBody!.linearDamping = 0.0
	}
	//タッチ移動
	override func touchesMoved(touches: Set<UITouch>, withEvent event: UIEvent?) {
		
		var location: CGPoint!
		for touch in touches {
			location = touch.locationInNode(self)
		}
		//移動角度
		let	radian = (atan2(location.y-self.tapPoint.y, location.x-self.tapPoint.x))
		let angle = radian * 180 / CGFloat(M_PI)
		if angle > -90 && angle < 90 {
			if self.moving == false || self.playerDirection != .Right {
				self.moveToRight()	//右
			}
		}
		else {
			if self.moving == false || self.playerDirection != .Left{
				self.moveToLeft()	//左
			}
		}
	}
	//タッチアップされたときに呼ばれる関数
	override func touchesEnded(touches: Set<UITouch>, withEvent event: UIEvent?) {
		self.moveStop()
	}
	
	
	
	
	//MARK: - シーンのアップデート時に呼ばれる関数
	override func update(currentTime: CFTimeInterval) {
		
		//MARK: プレイヤー移動
		if self.moving == true {
			var dx: CGFloat = 0
			var dy: CGFloat = 0
			if self.playerDirection == .Right {
				dx =  self.playerAcceleration
				dy =  0.0
			}
			else if self.playerDirection == .Left {
				dx =  -(self.playerAcceleration)
				dy =  0.0
			}
			self.playerNode.physicsBody!.applyForce(CGVectorMake(dx, dy))
			//速度制限
			if self.jumping == false && self.falling == false {
				//地面上の移動
				if self.playerNode.physicsBody!.velocity.dx > self.playerMaxVelocity {
					self.playerNode.physicsBody!.velocity.dx = self.playerMaxVelocity
				}
				else if self.playerNode.physicsBody!.velocity.dx < -(self.playerMaxVelocity) {
					self.playerNode.physicsBody!.velocity.dx = -(self.playerMaxVelocity)
				}
			} else {
				//空中での姿勢制御
				if self.playerNode.physicsBody!.velocity.dx > self.playerMaxVelocity / 2 {
					self.playerNode.physicsBody!.velocity.dx = self.playerMaxVelocity / 2
				}
				else if self.playerNode.physicsBody!.velocity.dx < -(self.playerMaxVelocity / 2) {
					self.playerNode.physicsBody!.velocity.dx = -(self.playerMaxVelocity / 2)
				}
			}
		}
		//MARK: 画面をスクロールさせる
		//シーン上でのプレイヤーの座標をbaseNodeからの位置に変換
		let PlayerPt = self.convertPoint(self.playerNode.position, fromNode: self.baseNode)
		//シーン上でプレイヤー位置を基準にしてbaseNodeの位置を変更する
		var	x = self.baseNode.position.x - PlayerPt.x + self.charXOffset
		var	y = self.baseNode.position.y - PlayerPt.y + self.charYOffset
		//スクロール制限
		if x <= -(self.allScreenSize.width - self.size.width) {
			x = -(self.allScreenSize.width - self.size.width)
		}
		if x > 0 {
			x = 0
		}
		if y <= -(self.allScreenSize.height - self.size.height) {
			y = -(self.allScreenSize.height - self.size.height)
		}
		if y > 0 {
			y = 0
		}
		self.baseNode.position = CGPointMake(x, y)
		self.backScrNode.position = CGPointMake(x/4, y)
		
		
		//MARK: 落下
		if self.playerNode.physicsBody?.velocity.dy < -9.8 && self.falling == false {
			self.jumping = false
			self.falling = true
			self.playerNode.physicsBody!.collisionBitMask = NodeName.frame_ground.category()|NodeName.frame_floor.category()
			self.playerNode.physicsBody!.contactTestBitMask = NodeName.frame_floor.category()|NodeName.frame_ground.category()
			if self.playerDirection == .Left {
				self.stopTextureAnimation(self.playerNode, name: "left_falling1")
			}
			else {
				self.stopTextureAnimation(self.playerNode, name: "right_falling1")
			}
		}
		
	}
	
	//MARK: - すべてのアクションと物理シミュレーション処理後、1フレーム毎に呼び出される
	override func didSimulatePhysics() {
		
	}
	
	//MARK: - 接触判定
	func didBeginContact(contact: SKPhysicsContact) {
		
		//着地
		//当たり判定戻す
		self.playerNode.physicsBody!.collisionBitMask = NodeName.frame_ground.category()|NodeName.frame_floor.category()
		self.playerNode.physicsBody!.contactTestBitMask = 0
		
		self.jumping = false
		self.falling = false
		
		if self.moving {
			//移動中
			if self.playerDirection == .Right {
				self.moveToRight()
			} else if self.playerDirection == .Left {
				self.moveToLeft()
			}
		}
		else {
			//停止
			self.moveStop()
		}
	}
	
	func didEndContact(contact: SKPhysicsContact) {
		
	}
	
	
	//==========================================================
	//MARK: - テクスチャアニメーション
	//==========================================================
	//開始
	func startTextureAnimation(node: SKSpriteNode, names: [String]) {
		node.removeActionForKey("textureAnimation")
		var ary: [SKTexture] = []
		for name in names {
			ary.append(SKTexture(imageNamed: name))
		}
		let action = SKAction.animateWithTextures(ary, timePerFrame: 0.1, resize: true, restore: false)
		node.runAction(SKAction.repeatActionForever(action), withKey: "textureAnimation")
	}
	
	//停止
	func stopTextureAnimation(node: SKSpriteNode, name: String) {
		//キャラクター
		node.removeActionForKey("textureAnimation")
		node.texture = SKTexture(imageNamed: name)
	}
	
}
